#!/usr/bin/python3.4

import argparse
import ipaddress
import json
import os
import subprocess
import sys

from ruamel import yaml

parser = argparse.ArgumentParser(description="Setup port forwards between docker networks.")
parser.add_argument('-c', '--config', metavar='PATH', default='/etc/nerve/docker-forwarder.yaml',
                    help="Use an alternative config file (default: %(default)s).")
parser.add_argument('-d', '--dry', default=False, action='store_true',
                    help="Do not actually do anything.")
parser.add_argument('action', choices=['start', 'stop'], help='Wether to add or remove iptables rules.')
args = parser.parse_args()

if not os.path.exists(args.config):
    print('%s: No such file or directory.' % args.config)
    sys.exit(0)

try:
    with open(args.config) as stream:
        data = yaml.safe_load(stream)
except Exception as e:
    print('Could not read %s: %s' % (args.config, e))
    sys.exit(1)


def run(cmd):
    if args.dry:
        print(' '.join(cmd))
    else:
        subprocess.call(cmd)


def get_network(network):
    if not networks.get(network):
        cmd = ['docker', 'network', 'inspect', network]
        networks[network] = json.loads(subprocess.check_output(cmd).decode('utf-8'))[0]
    return networks[network]


def get_addr(network, container):
    try:
        network = get_network(network)
    except Exception:
        print('%s: Could not get docker network.' % network)
        return
    #print(json.dumps(network, indent=4))

    if container:
        # get address of container
        for _container, container_data in network['Containers'].items():
            if container_data['Name'] == container:
                # IPv4Address has a subnet, so we have to get the raw ip
                # see also: https://docs.python.org/3/library/ipaddress.html#ipaddress.IPv4Interface
                addr = container_data['IPv4Address']
                return str(ipaddress.ip_interface(addr).ip)

        print('%s: Could not find container.' % container)
    else:
        return network['IPAM']['Config'][0]['Subnet']


networks = {}

for forward in data:
    comment = forward.get('comment', 'nerve-docker-port-forwarder')
    source = forward.get('source')
    dest = forward.get('destination')
    protocol = forward.get('protocol', 'tcp')
    port = forward.get('port')  # NOTE: still an int if it's a single port, or None if not specified

    if source is None or dest is None:
        print('Error: "source" and "destination" are a required values for every rule.')
        continue
    if not source.get('network'):
        print('Error: network is a required key for "source" and "destination".')
        continue

    source_network = source['network']
    source_container = source.get('container')
    dest_network = dest['network']
    dest_container = dest.get('container')

    source_addr = get_addr(source_network, source_container)
    if not source_addr:
        print('Could not find source address - skipping.')
        continue

    dest_addr = get_addr(dest_network, dest_container)
    if not dest_addr:
        print('Could not find destination address - skipping.')
        continue

    if args.action == 'start':
        iptables = ['iptables', '-I', 'FORWARD', ]
    else:
        iptables = ['iptables', '-D', 'FORWARD', ]

    # -I FORWARD 1 --source 172.17.0.0/16 --destination 172.18.0.4 -m tcp -p tcp --dport 8086 -j ACCEPT
    # -I FORWARD 1 --source 172.18.0.4 --destination 172.17.0.0/16 -m tcp -p tcp --sport 8086 -j ACCEPT
    incoming = iptables + [
        '-j', 'ACCEPT', '-m', 'comment', '--comment', comment,
        '--source', source_addr, '--destination', dest_addr,
    ]
    if port:
        incoming += ['-m', protocol, '-p', protocol, '--dport', str(port)]
    run(incoming)

    outgoing = iptables + [
        '-j', 'ACCEPT',
        '-m', 'conntrack', '--ctstate', 'RELATED,ESTABLISHED',
        '-m', 'comment', '--comment', comment,
        '--source', dest_addr, '--destination', source_addr,
    ]
    if port:
        outgoing += ['-m', protocol, '-p', protocol, '--sport', str(port)]
    run(outgoing)
